package dao;

import database.FixrateDatabase;
import dto.Order;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import static database.Database.close;

public class OrderDao {

    private FixrateDatabase fdb = new FixrateDatabase();
    private Connection connection = null;
    private ResultSet res = null;
    private PreparedStatement preparedStatement = null;
    private Order order = null;

    /**
     * Creates order object with order id, depositor id, and bank id
     * @param orderId String
     * @return Order object
     */
    public Order getOrder (String orderId) {
        String sqlQuery =
                "SELECT fixrate.cached_order.id," +
                        "fixrate.cached_order.depositor_id," +
                        "fixrate.cached_order.bank_id " +
                        "FROM fixrate.cached_order " +
                        "WHERE fixrate.cached_order.id = ?;";

        try {
            connection = fdb.connectToFixrateDB();
            preparedStatement = connection.prepareStatement(sqlQuery);
            preparedStatement.setString(1,orderId);
            res = preparedStatement.executeQuery();

            if (res.next()) {
                order = new Order();
                order.setOrderId(res.getString("id"));
                order.setUserId(res.getString("depositor_id"));
                order.setBankId(res.getString("bank_id"));
            }
            return order;
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } finally {
            close(connection,preparedStatement,res);
        }
        return null;
    }

    /**
     * This method creates an Order object, with user id, and order id
     * establishes connection to database
     * executes sql query
     * @param user_id String
     * @param order_id String
     * @return Order object
     */
    public Order getOrderInfo (String user_id, String order_id) {
        String query =
                "SELECT c_user.first_name, c_user.last_name, v_org_name.name AS org_name, bank.name AS bank_name, c_order.depositor_id AS organisation_id , c_order.bank_id AS bank_id " +
                    "FROM fixrate.cached_order c_order " +
                    "JOIN fixrate.v_organisation_name v_org_name ON c_order.depositor_id = v_org_name.id " +
                    "JOIN fixrate.cached_bank bank ON c_order.bank_id = bank.id " +
                    "JOIN fixrate.cached_user c_user WHERE c_user.id = ? " +
                    "AND c_order.id = ?;";

        try {
            connection = fdb.connectToFixrateDB();
            preparedStatement = connection.prepareStatement(query);
            preparedStatement.setString(1,user_id);
            preparedStatement.setString(2,order_id);
            res = preparedStatement.executeQuery();

            if (res.next()) {
                order = new Order();
                order.setDepositorFirstname(res.getString("first_name"));
                order.setDepositorLastname(res.getString("last_name"));
                order.setOrganisationName(res.getString("org_name"));
                order.setBankName(res.getString("bank_name"));
                order.setOrganisationId(res.getString("organisation_id"));
                order.setBankId(res.getString("bank_id"));
            }
            return order;
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } finally {
            close(connection,preparedStatement,res);
        }
        return null;
    }


    /**
     * This method finds information for one rate change on order, based on order id
     * establishes connection to database
     * executes sql query
     * @param order_id String
     * @return Order object
     */
    public Order getRateChangeInfo (String order_id) {
        String sqlQuery =
                "SELECT cb.name AS bank_name, depo.name AS org_name, rate.depositor_id AS depositor_id " +
                        "FROM fixrate.cached_interest_rate_change rate " +
                        "JOIN fixrate.cached_bank cb ON rate.bank_id = cb.id " +
                        "JOIN fixrate.cached_depositor depo ON rate.depositor_id = depo.id " +
                        "WHERE rate.id = ?;";

        try {
            connection = fdb.connectToFixrateDB();
            preparedStatement = connection.prepareStatement(sqlQuery);
            preparedStatement.setString(1,order_id);
            res = preparedStatement.executeQuery();
            System.out.println("res: " + res);
            if (res.next()) {
                order = new Order();
                order.setBankName(res.getString("bank_name"));
                order.setOrganisationName(res.getString("org_name"));
                order.setOrganisationId(res.getString("depositor_id"));
            }
            return order;

        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } finally {
            close(connection,preparedStatement,res);
        }
        return null;
    }

    /**
     * Gets an order object for cancelled orders
     * @param order_id String
     * @return Order object
     */
    public Order getCancelledOrderInfo (String order_id) {
        String sqlQuery =
                "SELECT cb.name AS bank_name, cd.name AS org_name, co.depositor_id AS organisation_id, co.bank_id AS bank_id " +
                        "FROM fixrate.cached_order co " +
                        "JOIN fixrate.cached_bank cb ON co.bank_id = cb.id " +
                        "JOIN fixrate.cached_depositor cd ON co.depositor_id = cd.id " +
                        "WHERE co.id = ?" +
                        "AND co.order_state = ?;";

        try {
            connection = fdb.connectToFixrateDB();
            preparedStatement = connection.prepareStatement(sqlQuery);
            preparedStatement.setString(1,order_id);
            preparedStatement.setString(2,"CANCELED");
            res = preparedStatement.executeQuery();

            if (res.next()) {
                order = new Order();
                order.setBankName(res.getString("bank_name"));
                order.setOrganisationName(res.getString("org_name"));
                order.setOrganisationId(res.getString("organisation_id"));
                order.setBankId(res.getString("bank_id"));
            }
            return order;
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } finally {
            close(connection,preparedStatement,res);
        }
        return null;
    }

    /**
     * Method to get an order object where depositor sends termination request
     * @param process_id String
     * @return Order object
     */
    public Order getDepositTerminationRequestInfo (String process_id) {
        String query =
                "SELECT p.user_full_name AS name, p.organisation_name AS org_name, d.bank_id AS bank_id " +
                        "FROM fixrate.cached_deposit d " +
                        "JOIN fixrate.cached_process_history p ON d.id = p.process_id " +
                        "WHERE d.id = ? AND p.type = ?;";


        try {
            connection = fdb.connectToFixrateDB();
            preparedStatement = connection.prepareStatement(query);
            preparedStatement.setString(1,process_id);
            preparedStatement.setString(2,"DEPOSIT_CONDITIONAL_TERMINATION_REQUESTED");
            //preparedStatement.setString(2,"DEPOSIT_TERMINATION_REQUESTED");
            res = preparedStatement.executeQuery();
            if (res.next()) {
                order = new Order();
                order.setDepositorFullname(res.getString("name"));
                order.setOrganisationName(res.getString("org_name"));
                order.setBankId(res.getString("bank_id")); // sendes til bank
            }
            return order;
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } finally {
            close(connection,preparedStatement,res);
        }
        return null;
    }

    /**
     * Method to get an order object where bank has confirmed termination of deposit
     * @param process_id String
     * @return Order object
     */
    public Order getDepositTerminationConfirmedInfo (String process_id) {
        String query =
                "SELECT p.user_full_name AS bank_user_name, p.organisation_name AS bank_name, d.depositor_id AS organisation_id, d.bank_id, v_organisation_name.name AS org_name " +
                "FROM fixrate.cached_deposit d " +
                "JOIN fixrate.cached_process_history p ON d.id = p.process_id " +
                "JOIN fixrate.v_organisation_name ON d.depositor_id = v_organisation_name.id " +
                "WHERE d.id = ? AND p.type = ?;";
        try {
            connection = fdb.connectToFixrateDB();
            preparedStatement = connection.prepareStatement(query);
            preparedStatement.setString(1,process_id);
            preparedStatement.setString(2,"DEPOSIT_TERMINATION_CONFIRMED");
            res = preparedStatement.executeQuery();
            if (res.next()) {
                order = new Order();
                order.setDepositorFullname(res.getString("bank_user_name"));
                order.setBankName(res.getString("bank_name"));
                order.setOrganisationId(res.getString("organisation_id"));//sendes til innskyter
                order.setBankId(res.getString("bank_id"));
                order.setOrganisationName(res.getString("org_name"));
            }
            return order;
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } finally {
            close(connection,preparedStatement,res);
        }
        return null;
    }
}
