import React, { useState } from 'react';
import {
  Text,
  View,
  StyleSheet,
  Modal,
  TouchableHighlight,
  Button,
  Image,
  Platform,
  Alert
} from 'react-native';
import { useDispatch, useSelector } from 'react-redux';
import { TouchableOpacity } from 'react-native-gesture-handler';
import Constants from 'expo-constants';
import { Ionicons } from '@expo/vector-icons';
import * as LocalAuthentication from 'expo-local-authentication';

import * as actions from '../../../store/actions/action';

import MainNavigator from '../../../navigations/MainNavigator';
import SecureStoreHandler from '../../../utils/secureStore';

const LocalAuth = () => {
  const [modalVisible, setModalVisible] = useState(false);
  const [failedCount, setFailedCount] = useState(0);

  const biometricEnabled = useSelector(state => state.init.loading.biometric);
  const authenticated = useSelector(state => state.init.authenticated);
  const reAuthenticated = useSelector(state => state.init.reAuth);
  const expoToken = useSelector(state => state.init.loading.deviceToken);
  const jwt = useSelector(state => state.init.jwt);
  const dispatch = useDispatch();

  const clearState = () => {
    setFailedCount(0);
  };

  const scanFingerPrint = async () => {
    try {
      const result = await LocalAuthentication.authenticateAsync({
        promptMessage: 'Skriv passord',
        fallbackLabel: 'Bruk Passord'
      });

      if (result.success) {
        setModalVisible(false);
        SecureStoreHandler.readStore('userJWTs', expoToken, dispatch);
      }

      if (result.error === 'authentication_failed') {
        const updated = failedCount + 1;
        setFailedCount(updated);
      }
      if (result.error === 'lockout') {
        Alert.alert('Vennligst logg inn på nytt med bankID.');
        dispatch(actions.logOut(expoToken, true));
      }
    } catch (e) {
      console.log(e);
    }
  };

  const AndroidModal = () => {
    return (
      <Modal
        animationType="fade"
        transparent={true}
        visible={modalVisible}
        onShow={scanFingerPrint}
      >
        <View style={modalStyle.centeredView}>
          <View style={modalStyle.modalView}>
            <Text style={modalStyle.loginText}>Logg inn med fingeravtrykk</Text>
            <Ionicons
              name="md-finger-print"
              size={70}
              color={'darkgray'}
              style={{ marginTop: 20 }}
            />

            <View style={{ marginBottom: 20 }}>
              {failedCount > 0 && (
                <Text style={modalStyle.failedCountText}>
                  Autentisering mislykket. Vennligst prøv igjen.
                </Text>
              )}
            </View>

            <TouchableHighlight
              style={modalStyle.cancelButton}
              onPress={() => {
                setModalVisible(false);
                LocalAuthentication.cancelAuthenticate();
              }}
            >
              <Text style={modalStyle.cancelButtonText}>Avbryt</Text>
            </TouchableHighlight>
          </View>
        </View>
      </Modal>
    );
  };

  const BioEnabled = props => {
    const { scan } = props;
    if (modalVisible) {
      return <AndroidModal />;
    }
    return (
      <View style={{ alignItems: 'center' }}>
        <View style={{ width: '80%' }}>
          <Image
            source={require('../../../assets/images/fixrateLogo.png')}
            style={{ resizeMode: 'contain', width: 300, height: 300 }}
          />
        </View>
        <View style={styles.greeting}>
          <Text style={styles.greetText}>Velkommen tilbake</Text>
        </View>
        <View>
          <TouchableOpacity
            style={[
              styles.button,
              { borderColor: '#99C157', backgroundColor: '#99C157', shadowColor: '#82A44A' }
            ]}
            onPress={scan}
          >
            <Text style={styles.buttonText}>Logg inn</Text>
          </TouchableOpacity>
          <TouchableOpacity
            style={[
              styles.button,
              { borderColor: '#b23b3b', backgroundColor: '#b23b3b', shadowColor: '#8B0000' }
            ]}
            onPress={() => {
              Alert.alert('Logge ut?', 'Hvis du logger ut må du logge inn med bankID igjen', [
                { text: 'Nei' },
                { text: 'Ja', onPress: () => dispatch(actions.logOut(expoToken, true)) }
              ]);
            }}
          >
            <Text style={styles.buttonText}>Logg ut</Text>
          </TouchableOpacity>
        </View>
      </View>
    );
  };

  const BioDisabled = () => (
    <Button
      title={'Start autentisering'}
      onPress={() => {
        clearState();
        if (Platform.OS === 'android') {
          setModalVisible(!modalVisible);
        } else {
          scanFingerPrint();
        }
      }}
    />
  );

  let content;
  if (biometricEnabled) {
    content = (
      <BioEnabled
        scan={
          Platform.OS === 'android'
            ? () => {
                clearState();
                setModalVisible(true);
              }
            : () => scanFingerPrint()
        }
      />
    );
  } else {
    content = <BioDisabled />;
  }

  if (!authenticated) {
    return <View style={[styles.container]}>{content}</View>;
  }

  if (reAuthenticated) {
    SecureStoreHandler.setKeyInStore(true, 'userJWTs', jwt, expoToken, dispatch).catch(err =>
      dispatch(actions.logOut(expoToken, true))
    );
    dispatch(actions.setReAuth(false));
  }
  return <MainNavigator />;
};

const primaryText = 'MuseoSansRounded-500';
const primaryTextBold = 'MuseoSansRounded-900';

const styles = StyleSheet.create({
  container: {
    flex: 1,
    justifyContent: 'center',
    alignItems: 'center',
    paddingTop: Constants.statusBarHeight,
    padding: 8,
    backgroundColor: '#EEEEEE'
  },
  text: {
    alignSelf: 'center',
    fontSize: 22,
    paddingTop: 20
  },
  buttonContainer: {
    backgroundColor: '#EEEEEE',
    flexDirection: 'row',
    alignItems: 'center',
    marginTop: 20
  },
  button: {
    borderRadius: 12,
    width: 100,
    height: 50,
    justifyContent: 'center',
    alignItems: 'center',
    shadowOpacity: 1,
    elevation: 6,
    shadowOffset: { width: 1, height: 3 },
    backgroundColor: '#99C157',
    margin: 10
  },
  buttonText: {
    color: '#fff',
    fontFamily: primaryText,
    fontSize: 20
  },
  greeting: {
    marginBottom: 20
  },
  greetText: {
    fontFamily: primaryTextBold,
    fontSize: 34
  }
});

const modalStyle = StyleSheet.create({
  centeredView: {
    flex: 1,
    justifyContent: 'flex-end',
    flexDirection: 'column',
    alignItems: 'center',
    opacity: 0.99,
    backgroundColor: 'rgba(100,100,100, 0.5)'
  },
  modalView: {
    backgroundColor: '#fff',
    padding: 35,
    height: '50%',
    width: '100%',
    alignItems: 'center',
    shadowColor: '#000',
    shadowOffset: {
      width: 1,
      height: 2
    },
    shadowOpacity: 0.25,
    shadowRadius: 3.84,
    elevation: 5
  },
  cancelButton: {
    borderRadius: 12,
    borderColor: '#8B0000',
    borderWidth: 1,
    backgroundColor: '#FF0000',
    padding: 10,
    shadowColor: '#8B0000',
    shadowOpacity: 1,
    shadowRadius: 2,
    shadowOffset: { width: 1, height: 3 },
    elevation: 10,
    overflow: Platform.OS === 'android' && Platform.Version >= 21 ? 'hidden' : 'visible'
  },
  cancelButtonText: {
    color: '#fff',
    fontSize: 16,
    fontFamily: 'MuseoSansRounded-500'
  },
  failedCountText: {
    color: '#FF0000',
    fontSize: 14,
    fontFamily: 'MuseoSansRounded-500'
  },
  loginText: {
    fontFamily: 'MuseoSansRounded-500',
    margin: 10,
    fontSize: 20
  }
});

export default LocalAuth;
