import React, { useState } from 'react';
import {
  View,
  Text,
  StyleSheet,
  TouchableOpacity,
  Modal,
  TouchableHighlight,
  Platform,
  Alert
} from 'react-native';
import { useSelector, useDispatch } from 'react-redux';
import * as LocalAuthentication from 'expo-local-authentication';
import { Ionicons } from '@expo/vector-icons';

import * as actions from '../../../store/actions/action';

import SecureStoreHandler from '../../../utils/secureStore';

const PromptScreen = () => {
  const dispatch = useDispatch();
  const jwt = useSelector(state => state.init.jwt);
  const expoToken = useSelector(state => state.init.loading.deviceToken);

  const [modalVisible, setModalVisible] = useState(false);
  const [failedCount, setFailedCount] = useState(0);

  const scanFingerPrint = async () => {
    try {
      const result = await LocalAuthentication.authenticateAsync({
        promptMessage: 'Skriv passord',
        fallbackLabel: 'Bruk Passord'
      });

      if (result.success) {
        setModalVisible(false);
        SecureStoreHandler.setKeyInStore(true, 'userJWTs', jwt, expoToken, dispatch);
      }

      if (result.error === 'authentication_failed') {
        const updated = failedCount + 1;
        setFailedCount(updated);
      }
      if (result.error === 'lockout') {
        Alert.alert('Vennligst logg inn på nytt med bankID.');
        dispatch(actions.logOut(expoToken, true));
      }
    } catch (e) {
      console.log(e);
    }
  };

  const CustButton = props => {
    const { buttonText, color, press } = props;
    return (
      <TouchableOpacity style={[styles.button, { backgroundColor: color }]} onPress={press}>
        <Text style={styles.buttonText}>{buttonText}</Text>
      </TouchableOpacity>
    );
  };

  const AndroidModal = () => {
    return (
      <Modal
        animationType={'fade'}
        transparent={true}
        visible={modalVisible}
        onShow={scanFingerPrint}
      >
        <View style={modalStyle.centeredView}>
          <View style={modalStyle.modalView}>
            <Text style={modalStyle.loginText}>Logg inn med fingeravtrykk</Text>
            <Ionicons
              name="md-finger-print"
              size={70}
              color={'darkgray'}
              style={{ marginTop: 20 }}
            />

            <View style={{ marginBottom: 20 }}>
              {failedCount > 0 && (
                <Text style={modalStyle.failedCountText}>
                  Autentisering mislykket. Vennligst prøv igjen.
                </Text>
              )}
            </View>

            <TouchableHighlight
              style={modalStyle.cancelButton}
              onPress={() => {
                setModalVisible(false);
                LocalAuthentication.cancelAuthenticate();
              }}
            >
              <Text style={modalStyle.cancelButtonText}>Avbryt</Text>
            </TouchableHighlight>
          </View>
        </View>
      </Modal>
    );
  };

  if (modalVisible) {
    return <AndroidModal />;
  }
  return (
    <View style={styles.container}>
      <View style={styles.roundedCorners}>
        <Text style={styles.text}>
          Vil du bruke biometrisk innlogging ved fremtidige innlogginger?
        </Text>
        <View style={styles.buttonContainer}>
          <CustButton
            buttonText="Nei"
            color="#EFF4F9"
            press={() => {
              dispatch(actions.setPrompt(false, true, expoToken));
            }}
          />
          <CustButton
            buttonText="Ja"
            color="#99C157"
            press={() => {
              setFailedCount(0);
              if (Platform.OS === 'android') {
                setModalVisible(!modalVisible);
              } else {
                scanFingerPrint();
              }
            }}
          />
        </View>
      </View>
    </View>
  );
};

const primaryText = 'MuseoSansRounded-500';
const primaryTextBold = 'MuseoSansRounded-900';

const modalStyle = StyleSheet.create({
  centeredView: {
    flex: 1,
    justifyContent: 'flex-end',
    flexDirection: 'column',
    alignItems: 'center',
    opacity: 0.99,
    backgroundColor: 'rgba(100,100,100, 0.5)'
  },
  modalView: {
    backgroundColor: '#fff',
    padding: 35,
    height: '50%',
    width: '100%',
    alignItems: 'center',
    shadowColor: '#000',
    shadowOffset: {
      width: 1,
      height: 2
    },
    shadowOpacity: 0.25,
    shadowRadius: 3.84,
    elevation: 5
  },
  cancelButton: {
    borderRadius: 12,
    borderColor: '#b23b3b',
    backgroundColor: '#b23b3b',
    shadowColor: '#8B0000',
    borderWidth: 1,
    padding: 10,
    shadowOpacity: 1,
    shadowRadius: 2,
    shadowOffset: { width: 1, height: 3 },
    elevation: 10,
    overflow: Platform.OS === 'android' && Platform.Version >= 21 ? 'hidden' : 'visible'
  },
  cancelButtonText: {
    color: '#fff',
    fontSize: 16,
    fontFamily: 'MuseoSansRounded-500'
  },
  failedCountText: {
    color: '#FF0000',
    fontSize: 14,
    fontFamily: 'MuseoSansRounded-500'
  },
  loginText: {
    fontFamily: 'MuseoSansRounded-500',
    margin: 10,
    fontSize: 20
  }
});

const styles = StyleSheet.create({
  wrapper: {
    flex: 1,
    flexDirection: 'column'
  },
  text: {
    fontFamily: primaryText,
    fontSize: 20,
    textAlign: 'center'
  },
  container: {
    backgroundColor: '#EEEEEE',
    flex: 1,
    flexDirection: 'column',
    justifyContent: 'center',
    alignItems: 'center'
  },
  buttonContainer: {
    backgroundColor: 'white',
    flexDirection: 'row',
    justifyContent: 'space-evenly',
    marginTop: 20
  },
  customButton: {
    borderColor: '#99C157',
    backgroundColor: '#99C157',
    shadowColor: '#82A44A',
    borderRadius: 12,
    margin: 8,
    padding: 10,
    borderWidth: 1,
    shadowOpacity: 1,
    elevation: 6,
    shadowOffset: { width: 1, height: 3 }
  },
  button: {
    borderRadius: 12,
    width: 100,
    height: 50,
    justifyContent: 'center',
    alignItems: 'center',
    shadowOpacity: 1,
    elevation: 6,
    shadowOffset: { width: 1, height: 3 }
  },
  buttonText: {
    fontFamily: primaryText,
    fontSize: 24
  },
  roundedCorners: {
    borderRadius: 12,
    backgroundColor: 'white',
    margin: 8,
    padding: 20,
    shadowColor: '#4A4646',
    shadowOpacity: 0.8,
    elevation: 6,
    shadowOffset: { width: 1, height: 5 }
  }
});
export default PromptScreen;
